@@ -206,16 +206,12 @@ testExternalGet (int flags)
   return 0;
 }
 
-GCRY_THREAD_OPTION_PTHREAD_IMPL;
 
 int
 main (int argc, char *const *argv)
 {
   unsigned int errorCount = 0;
 
-  gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
-  if (!gcry_check_version (GCRYPT_VERSION))
-    abort ();
   if (0 != curl_global_init (CURL_GLOBAL_ALL))
     {
       fprintf (stderr, "Error: %s\n", strerror (errno));
